/*
 * Created on May 1st 2023
 *
 * Copyright (C) 2023 Welltested AI - All Rights Reserved
 * The code in this file is Intellectutal Property of Welltested AI.
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'dart:math';

import 'package:analysis_server_lib/analysis_server_lib.dart';
import 'package:welltested/src/utilities/typedef.dart';
import 'package:welltested/src/visitors/unit_test_visitor.dart';

extension ReplaceList on List<ReplaceModel> {
  List<ReplaceModel> get extractUniqueAndUnoverlapped {
    final List<ReplaceModel> uniqueModels = [];
    this.forEach((element) {
      if (uniqueModels.indexWhere((unique) =>
              element.offset == unique.offset &&
              element.endOffset == unique.endOffset) ==
          -1) {
        uniqueModels.add(element);
      }
    });
    uniqueModels.removeWhere((element) {
      for (int i = 0; i < uniqueModels.length; i++) {
        if (element.offset > uniqueModels[i].offset &&
            element.endOffset < uniqueModels[i].endOffset) {
          return true;
        }
      }
      return false;
    });
    return uniqueModels;
  }
}

extension SourceEditSequence on List<SourceEdit> {
  List<SourceEdit> get filterDuplicationReplacements {
    final List<SourceEdit> filteredList = [];
    forEach((edit) {
      if (filteredList.indexWhere(
              (element) => element.replacement == edit.replacement) ==
          -1) {
        filteredList.add(edit);
      }
    });
    return filteredList;
  }
}

extension ArgsExtension on List<String> {
  OperationKind get operationKind {
    if (length < 1) return OperationKind.None;
    switch (this[0]) {
      case 'generate':
        return OperationKind.generate;
      case 'save':
        return OperationKind.save;
      default:
        return OperationKind.None;
    }
  }

  TestKind get testKind {
    if (length < 2) return TestKind.None;
    switch (this[1]) {
      case 'unit':
        return TestKind.Unit;
      default:
        return TestKind.None;
    }
  }
}

extension ReadableStringExtension on List<String>? {
  String humanPreferredString(int visibleCount) {
    final stringList = this;

    if (stringList == null) return '';

    final visibleSize = visibleCount == -1
        ? stringList.length
        : min(visibleCount, stringList.length);

    final size = stringList.length;

    if (size == 0) return '';
    if (size == 1) return stringList.first;
    if (size == 2) return stringList.join(' and ');

    var outputString = '';
    if (size > visibleSize) {
      final subList = stringList.sublist(0, visibleSize);
      outputString += subList.join(', ');
      outputString += ', and others';
    } else if (size == visibleSize) {
      final lastItem = stringList.last;
      final subList = stringList.sublist(0, visibleSize - 1);
      outputString += subList.join(', ');
      outputString += ', and ${lastItem}';
    }
    return outputString;
  }
}
